@@ -28,7 +28,7 @@ _encoding_tests = [
     # <-- add encodings here
 ]
 
-def decode(enc, encoding=None):
+def decode(enc, encoding=None, errors='strict'):
     """decode(encoded_string) returns an unencoded unicode string
     """
     if enc is None or type(enc) is ustr:
@@ -41,7 +41,7 @@ def decode(enc, encoding=None):
 
     for encoding in encoding_tests:
         try:
-            return enc.decode(encoding)
+            return enc.decode(encoding, errors)
         except:
             pass
     # this shouldn't ever happen... FIXME
@@ -64,10 +64,10 @@ def mkpath(path, encoding=None):
     return encode(path, encoding=encoding)
 
 
-def read(filename, size=-1, encoding=None):
+def read(filename, size=-1, encoding=None, errors='strict'):
     """Read filename and return contents"""
     with xopen(filename, 'rb') as fh:
-        return fread(fh, size=size, encoding=encoding)
+        return fread(fh, size=size, encoding=encoding, errors=errors)
 
 
 def write(path, contents, encoding=None):
@@ -77,9 +77,9 @@ def write(path, contents, encoding=None):
 
 
 @interruptable
-def fread(fh, size=-1, encoding=None):
+def fread(fh, size=-1, encoding=None, errors='strict'):
     """Read from a filehandle and retry when interrupted"""
-    return decode(fh.read(size), encoding=encoding)
+    return decode(fh.read(size), encoding=encoding, errors=errors)
 
 
 @interruptable
@@ -116,17 +116,14 @@ def start_command(cmd, cwd=None, add_env=None,
         env = os.environ.copy()
         env.update(add_env)
 
-    if PY3:
-        # Python3 on windows always goes through list2cmdline() internally inside
-        # of subprocess.py so we must provide unicode strings here otherwise
-        # Python3 breaks when bytes are provided.
-        #
-        # Additionally, the preferred usage on Python3 is to pass unicode
-        # strings to subprocess.  Python will automatically encode into the
-        # default encoding (utf-8) when it gets unicode strings.
-        cmd = [decode(c) for c in cmd]
-    else:
-        cmd = [encode(c) for c in cmd]
+    # Python3 on windows always goes through list2cmdline() internally inside
+    # of subprocess.py so we must provide unicode strings here otherwise
+    # Python3 breaks when bytes are provided.
+    #
+    # Additionally, the preferred usage on Python3 is to pass unicode
+    # strings to subprocess.  Python will automatically encode into the
+    # default encoding (utf-8) when it gets unicode strings.
+    cmd = prep_for_subprocess(cmd)
 
     if WIN32 and cwd == getcwd():
         # Windows cannot deal with passing a cwd that contains unicode
@@ -141,6 +138,16 @@ def start_command(cmd, cwd=None, add_env=None,
                             universal_newlines=universal_newlines, **extra)
 
 
+def prep_for_subprocess(cmd):
+    """Decode on Python3, encode on Python2"""
+    # See the comment in start_command()
+    if PY3:
+        cmd = [decode(c) for c in cmd]
+    else:
+        cmd = [encode(c) for c in cmd]
+    return cmd
+
+
 @interruptable
 def communicate(proc):
     return proc.communicate()
