@@ -1,25 +1,28 @@
 from __future__ import division, absolute_import, unicode_literals
 
+import re
+
 from PyQt4 import QtCore
 from PyQt4 import QtGui
 from PyQt4.QtCore import Qt, SIGNAL
 
+from cola import actions
 from cola import cmds
 from cola import core
+from cola import gitcfg
 from cola import gitcmds
 from cola import gravatar
 from cola import qtutils
-from cola.cmds import run
 from cola.i18n import N_
 from cola.models import main
 from cola.models import selection
 from cola.qtutils import add_action
 from cola.qtutils import create_action_button
 from cola.qtutils import create_menu
-from cola.qtutils import DiffSyntaxHighlighter
+from cola.qtutils import RGB, make_format
 from cola.qtutils import options_icon
 from cola.widgets import defs
-from cola.widgets.text import MonoTextView
+from cola.widgets.text import VimMonoTextView
 from cola.compat import ustr
 
 
@@ -27,10 +30,102 @@ COMMITS_SELECTED = 'COMMITS_SELECTED'
 FILES_SELECTED = 'FILES_SELECTED'
 
 
-class DiffTextEdit(MonoTextView):
+class DiffSyntaxHighlighter(QtGui.QSyntaxHighlighter):
+    """Implements the diff syntax highlighting"""
+
+    INITIAL_STATE = -1
+    DIFFSTAT_STATE = 0
+    DIFF_FILE_HEADER_STATE = 1
+    DIFF_STATE = 2
+    SUBMODULE_STATE = 3
+
+    DIFF_FILE_HEADER_START_RGX = re.compile(r'diff --git a/.* b/.*')
+    DIFF_HUNK_HEADER_RGX = re.compile(r'(?:@@ -[0-9,]+ \+[0-9,]+ @@)|'
+                                      r'(?:@@@ (?:-[0-9,]+ ){2}\+[0-9,]+ @@@)')
+    BAD_WHITESPACE_RGX = re.compile(r'\s+$')
+
+    def __init__(self, doc, whitespace=True):
+        QtGui.QSyntaxHighlighter.__init__(self, doc)
+        self.whitespace = whitespace
+        self.enabled = True
+
+        cfg = gitcfg.current()
+        self.color_text = RGB(cfg.color('text', '030303'))
+        self.color_add = RGB(cfg.color('add', 'd2ffe4'))
+        self.color_remove = RGB(cfg.color('remove', 'fee0e4'))
+        self.color_header = RGB(cfg.color('header', 'bbbbbb'))
+
+        self.diff_header_fmt = make_format(fg=self.color_header)
+        self.bold_diff_header_fmt = make_format(fg=self.color_header,
+                                                  bold=True)
+
+        self.diff_add_fmt = make_format(fg=self.color_text,
+                                          bg=self.color_add)
+        self.diff_remove_fmt = make_format(fg=self.color_text,
+                                             bg=self.color_remove)
+        self.bad_whitespace_fmt = make_format(bg=Qt.red)
+
+    def set_enabled(self, enabled):
+        self.enabled = enabled
+
+    def highlightBlock(self, text):
+        if not self.enabled:
+            return
+
+        text = ustr(text)
+        if not text:
+            return
+
+        state = self.previousBlockState()
+        if state == self.INITIAL_STATE:
+            if text.startswith('Submodule '):
+                state = self.SUBMODULE_STATE
+            else:
+                state = self.DIFFSTAT_STATE
+
+        if state == self.DIFFSTAT_STATE:
+            if self.DIFF_FILE_HEADER_START_RGX.match(text):
+                state = self.DIFF_FILE_HEADER_STATE
+                self.setFormat(0, len(text), self.diff_header_fmt)
+            elif self.DIFF_HUNK_HEADER_RGX.match(text):
+                state = self.DIFF_STATE
+                self.setFormat(0, len(text), self.bold_diff_header_fmt)
+            elif '|' in text:
+                i = text.index('|')
+                self.setFormat(0, i, self.bold_diff_header_fmt)
+                self.setFormat(i, len(text) - i, self.diff_header_fmt)
+            else:
+                self.setFormat(0, len(text), self.diff_header_fmt)
+        elif state == self.DIFF_FILE_HEADER_STATE:
+            if self.DIFF_HUNK_HEADER_RGX.match(text):
+                state = self.DIFF_STATE
+                self.setFormat(0, len(text), self.bold_diff_header_fmt)
+            else:
+                self.setFormat(0, len(text), self.diff_header_fmt)
+        elif state == self.DIFF_STATE:
+            if self.DIFF_FILE_HEADER_START_RGX.match(text):
+                state = self.DIFF_FILE_HEADER_STATE
+                self.setFormat(0, len(text), self.diff_header_fmt)
+            elif self.DIFF_HUNK_HEADER_RGX.match(text):
+                self.setFormat(0, len(text), self.bold_diff_header_fmt)
+            elif text.startswith('-'):
+                self.setFormat(0, len(text), self.diff_remove_fmt)
+            elif text.startswith('+'):
+                self.setFormat(0, len(text), self.diff_add_fmt)
+                if self.whitespace:
+                    m = self.BAD_WHITESPACE_RGX.search(text)
+                    if m is not None:
+                        i = m.start()
+                        self.setFormat(i, len(text) - i,
+                                       self.bad_whitespace_fmt)
+
+        self.setCurrentBlockState(state)
+
+
+class DiffTextEdit(VimMonoTextView):
     def __init__(self, parent, whitespace=True):
 
-        MonoTextView.__init__(self, parent)
+        VimMonoTextView.__init__(self, parent)
         # Diff/patch syntax highlighter
         self.highlighter = DiffSyntaxHighlighter(self.document(),
                                                  whitespace=whitespace)
@@ -92,24 +187,41 @@ class DiffEditor(DiffTextEdit):
                 self.apply_selection, Qt.Key_S)
 
         self.action_revert_selection = qtutils.add_action(self, '',
-                self.revert_selection)
-        self.action_revert_selection.setIcon(qtutils.theme_icon('edit-undo.svg'))
+                self.revert_selection, Qt.ControlModifier + Qt.Key_U)
+        icon = qtutils.theme_icon('edit-undo.svg')
+        self.action_revert_selection.setIcon(icon)
 
-        self.launch_editor = qtutils.add_action(self,
-                cmds.LaunchEditor.name(), run(cmds.LaunchEditor),
-                cmds.LaunchEditor.SHORTCUT,
-                'Return', 'Enter')
-        self.launch_editor.setIcon(qtutils.options_icon())
+        self.launch_editor = actions.launch_editor(self, 'Return', 'Enter')
+        self.launch_difftool = actions.launch_difftool(self)
+        self.stage_or_unstage = actions.stage_or_unstage(self)
 
-        self.launch_difftool = qtutils.add_action(self,
-                cmds.LaunchDifftool.name(), run(cmds.LaunchDifftool),
-                cmds.LaunchDifftool.SHORTCUT)
-        self.launch_difftool.setIcon(qtutils.git_icon())
+        # Emit up/down signals so that they can be routed by the main widget
+        self.move_down = actions.move_down(self)
+        self.move_up = actions.move_up(self)
 
         model.add_observer(model.message_diff_text_changed, self._emit_text)
 
+        self.selection_model = selection_model = selection.selection_model()
+        selection_model.add_observer(selection_model.message_selection_changed,
+                                     self._update)
+        self.connect(self, SIGNAL('update()'),
+                     self._update_callback, Qt.QueuedConnection)
+
         self.connect(self, SIGNAL('set_text(PyQt_PyObject)'), self.setPlainText)
 
+    def _update(self):
+        self.emit(SIGNAL('update()'))
+
+    def _update_callback(self):
+        enabled = False
+        s = self.selection_model.selection()
+        if s.modified and self.model.stageable():
+            if s.modified[0] in self.model.submodules:
+                pass
+            elif s.modified[0] not in main.model().unstaged_deleted:
+                enabled = True
+        self.action_revert_selection.setEnabled(enabled)
+
     def _emit_text(self, text):
         self.emit(SIGNAL('set_text(PyQt_PyObject)'), text)
 
@@ -132,6 +244,13 @@ class DiffEditor(DiffTextEdit):
         s = selection.selection()
         filename = selection.filename()
 
+        if self.model.stageable() or self.model.unstageable():
+            if self.model.stageable():
+                self.stage_or_unstage.setText(N_('Stage'))
+            else:
+                self.stage_or_unstage.setText(N_('Unstage'))
+            menu.addAction(self.stage_or_unstage)
+
         if s.modified and self.model.stageable():
             if s.modified[0] in main.model().submodules:
                 action = menu.addAction(qtutils.add_icon(),
@@ -176,7 +295,6 @@ class DiffEditor(DiffTextEdit):
 
                 self.action_apply_selection.setText(apply_text)
                 self.action_apply_selection.setIcon(qtutils.remove_icon())
-
                 menu.addAction(self.action_apply_selection)
 
         if self.model.stageable() or self.model.unstageable():
@@ -189,6 +307,12 @@ class DiffEditor(DiffTextEdit):
             # Removed files can still be diffed.
             menu.addAction(self.launch_difftool)
 
+        # Add the Previous/Next File actions, which improves discoverability
+        # of their associated shortcuts
+        menu.addSeparator()
+        menu.addAction(self.move_up)
+        menu.addAction(self.move_down)
+
         menu.addSeparator()
         action = menu.addAction(qtutils.theme_icon('edit-copy.svg'),
                                 N_('Copy'), self.copy)
