@@ -194,11 +194,11 @@ class MainView(standard.MainWindow):
 
         self.browse_recently_modified_action = add_action(self,
                 N_('Recently Modified Files...'),
-                recent.browse_recent_files, 'Shift+Ctrl+E')
+                recent.browse_recent_files, 'Ctrl+Shift+E')
 
         self.cherry_pick_action = add_action(self,
                 N_('Cherry-Pick...'),
-                guicmds.cherry_pick, 'Shift+Ctrl+C')
+                guicmds.cherry_pick, 'Ctrl+Shift+C')
 
         self.load_commitmsg_action = add_action(self,
                 N_('Load Commit Message...'), guicmds.load_commitmsg)
@@ -211,7 +211,7 @@ class MainView(standard.MainWindow):
         self.grep_action = add_action(self,
                 N_('Grep'), grep.grep, 'Ctrl+G')
         self.merge_local_action = add_action(self,
-                N_('Merge...'), merge.local_merge, 'Shift+Ctrl+M')
+                N_('Merge...'), merge.local_merge, 'Ctrl+Shift+M')
 
         self.merge_abort_action = add_action(self,
                 N_('Abort Merge...'), merge.abort_merge)
@@ -221,7 +221,7 @@ class MainView(standard.MainWindow):
         self.push_action = add_action(self,
                 N_('Push...'), remote.push, 'Ctrl+P')
         self.pull_action = add_action(self,
-                N_('Pull...'), remote.pull, 'Shift+Ctrl+P')
+                N_('Pull...'), remote.pull, 'Ctrl+Shift+P')
 
         self.open_repo_action = add_action(self,
                 N_('Open...'), guicmds.open_repo)
@@ -311,19 +311,6 @@ class MainView(standard.MainWindow):
         self.rebase_abort_action = add_action(self,
                 N_('Abort'), self.rebase_abort)
 
-        # Relayed actions
-        status_tree = self.statusdockwidget.widget().tree
-        self.addAction(status_tree.delete_untracked_files_action)
-
-        if not self.browser_dockable:
-            # These shortcuts conflict with those from the
-            # 'Browser' widget so don't register them when
-            # the browser is a dockable tool.
-            self.addAction(status_tree.revert_unstaged_edits_action)
-            self.addAction(status_tree.up_action)
-            self.addAction(status_tree.down_action)
-            self.addAction(status_tree.process_selection_action)
-
         self.lock_layout_action = add_action_bool(self,
                 N_('Lock Layout'), self.set_lock_layout, False)
 
@@ -477,6 +464,18 @@ class MainView(standard.MainWindow):
         self.connect(self.diffeditor, SIGNAL('diff_options_updated()'),
                      self.statuswidget.refresh)
 
+        self.connect(self.diffeditor, SIGNAL('move_down()'),
+                     self.statuswidget.move_down)
+
+        self.connect(self.diffeditor, SIGNAL('move_up()'),
+                     self.statuswidget.move_up)
+
+        self.connect(self.commitmsgeditor, SIGNAL('move_down()'),
+                     self.statuswidget.move_down)
+
+        self.connect(self.commitmsgeditor, SIGNAL('move_up()'),
+                     self.statuswidget.move_up)
+
         self.connect(self, SIGNAL('update()'),
                      self._update_callback, Qt.QueuedConnection)
 
@@ -549,6 +548,9 @@ class MainView(standard.MainWindow):
             # enables automatic line breaks
             self.commitmsgeditor.set_linebreak(value)
 
+        elif config == prefs.SORT_BOOKMARKS:
+            self.bookmarkswidget.reload_bookmarks()
+
         elif config == prefs.TEXTWIDTH:
             # text width used for line wrapping
             self.commitmsgeditor.set_textwidth(value)
@@ -635,6 +637,8 @@ class MainView(standard.MainWindow):
         self.rebase_continue_action.setEnabled(is_rebasing)
         self.rebase_skip_action.setEnabled(is_rebasing)
         self.rebase_abort_action.setEnabled(is_rebasing)
+        self.rename_branch_action.setEnabled(not self.model.is_empty_repository())
+        self.delete_branch_action.setEnabled(not self.model.is_empty_repository())
 
     def export_state(self):
         state = standard.MainWindow.export_state(self)
