@@ -30,13 +30,14 @@ def preferences(model=None, parent=None):
 
 
 class FormWidget(QtGui.QWidget):
+
     def __init__(self, model, parent, source='user'):
         QtGui.QWidget.__init__(self, parent)
         self.model = model
         self.config_to_widget = {}
         self.widget_to_config = {}
         self.source = source
-        self.config = gitcfg.instance()
+        self.config = gitcfg.current()
         self.defaults = {}
         self.setLayout(QtGui.QFormLayout())
 
@@ -105,6 +106,7 @@ class FormWidget(QtGui.QWidget):
 
 
 class RepoFormWidget(FormWidget):
+
     def __init__(self, model, parent, source):
         FormWidget.__init__(self, model, parent, source=source)
 
@@ -129,6 +131,11 @@ class RepoFormWidget(FormWidget):
         self.display_untracked = QtGui.QCheckBox()
         self.display_untracked.setChecked(True)
 
+        tooltip = N_('Detect conflict markers in unmerged files')
+        self.check_conflicts = QtGui.QCheckBox()
+        self.check_conflicts.setChecked(True)
+        self.check_conflicts.setToolTip(tooltip)
+
         self.add_row(N_('User Name'), self.name)
         self.add_row(N_('Email Address'), self.email)
         self.add_row(N_('Merge Verbosity'), self.merge_verbosity)
@@ -136,8 +143,10 @@ class RepoFormWidget(FormWidget):
         self.add_row(N_('Summarize Merge Commits'), self.merge_summary)
         self.add_row(N_('Show Diffstat After Merge'), self.merge_diffstat)
         self.add_row(N_('Display Untracked Files'), self.display_untracked)
+        self.add_row(N_('Detect Conflict Markers'), self.check_conflicts)
 
         self.set_config({
+            prefs.CHECKCONFLICTS: (self.check_conflicts, True),
             prefs.DIFFCONTEXT: (self.diff_context, 5),
             prefs.DISPLAY_UNTRACKED: (self.display_untracked, True),
             prefs.USER_NAME: (self.name, ''),
@@ -149,6 +158,7 @@ class RepoFormWidget(FormWidget):
 
 
 class SettingsFormWidget(FormWidget):
+
     def __init__(self, model, parent):
         FormWidget.__init__(self, model, parent)
 
@@ -174,6 +184,7 @@ class SettingsFormWidget(FormWidget):
         self.difftool = QtGui.QLineEdit()
         self.mergetool = QtGui.QLineEdit()
         self.keep_merge_backups = QtGui.QCheckBox()
+        self.sort_bookmarks = QtGui.QCheckBox()
         self.save_gui_settings = QtGui.QCheckBox()
 
         self.add_row(N_('Fixed-Width Font'), self.fixed_font)
@@ -186,6 +197,7 @@ class SettingsFormWidget(FormWidget):
         self.add_row(N_('Diff Tool'), self.difftool)
         self.add_row(N_('Merge Tool'), self.mergetool)
         self.add_row(N_('Keep *.orig Merge Backups'), self.keep_merge_backups)
+        self.add_row(N_('Sort bookmarks alphabetically'), self.sort_bookmarks)
         self.add_row(N_('Save GUI Settings'), self.save_gui_settings)
 
         self.set_config({
@@ -193,14 +205,16 @@ class SettingsFormWidget(FormWidget):
             prefs.TABWIDTH: (self.tabwidth, 8),
             prefs.TEXTWIDTH: (self.textwidth, 72),
             prefs.LINEBREAK: (self.linebreak, True),
+            prefs.SORT_BOOKMARKS: (self.sort_bookmarks, True),
             prefs.DIFFTOOL: (self.difftool, 'xxdiff'),
             prefs.EDITOR: (self.editor, os.getenv('VISUAL', 'gvim')),
-            prefs.HISTORY_BROWSER: (self.historybrowser, 'gitk'),
+            prefs.HISTORY_BROWSER: (self.historybrowser,
+                                    prefs.default_history_browser()),
             prefs.MERGE_KEEPBACKUP: (self.keep_merge_backups, True),
             prefs.MERGETOOL: (self.mergetool, 'xxdiff'),
         })
 
-        self.connect(self.fixed_font, SIGNAL('currentFontChanged(const QFont &)'),
+        self.connect(self.fixed_font, SIGNAL('currentFontChanged(QFont)'),
                      self.current_font_changed)
 
         self.connect(self.font_size, SIGNAL('valueChanged(int)'),
@@ -240,43 +254,37 @@ class PreferencesView(standard.Dialog):
 
         self.resize(600, 360)
 
-        self._tabbar = QtGui.QTabBar()
-        self._tabbar.setDrawBase(False)
-        self._tabbar.addTab(N_('All Repositories'))
-        self._tabbar.addTab(N_('Current Repository'))
-        self._tabbar.addTab(N_('Settings'))
+        self.tab_bar = QtGui.QTabBar()
+        self.tab_bar.setDrawBase(False)
+        self.tab_bar.addTab(N_('All Repositories'))
+        self.tab_bar.addTab(N_('Current Repository'))
+        self.tab_bar.addTab(N_('Settings'))
 
-        self._user_form = RepoFormWidget(model, self, source='user')
-        self._repo_form = RepoFormWidget(model, self, source='repo')
-        self._options_form = SettingsFormWidget(model, self)
+        self.user_form = RepoFormWidget(model, self, source='user')
+        self.repo_form = RepoFormWidget(model, self, source='repo')
+        self.options_form = SettingsFormWidget(model, self)
 
-        self._stackedwidget = QtGui.QStackedWidget()
-        self._stackedwidget.addWidget(self._user_form)
-        self._stackedwidget.addWidget(self._repo_form)
-        self._stackedwidget.addWidget(self._options_form)
+        self.stack_widget = QtGui.QStackedWidget()
+        self.stack_widget.addWidget(self.user_form)
+        self.stack_widget.addWidget(self.repo_form)
+        self.stack_widget.addWidget(self.options_form)
 
         self.close_button = QtGui.QPushButton(self)
         self.close_button.setText(N_('Close'))
         self.close_button.setIcon(qtutils.close_icon())
 
-        self._button_layt = QtGui.QHBoxLayout()
-        self._button_layt.setMargin(defs.no_margin)
-        self._button_layt.setSpacing(defs.spacing)
-        self._button_layt.addStretch()
-        self._button_layt.addWidget(self.close_button)
+        self.button_layout = qtutils.hbox(defs.no_margin, defs.spacing,
+                                          qtutils.STRETCH, self.close_button)
 
-        self._layt = QtGui.QVBoxLayout()
-        self._layt.setMargin(defs.margin)
-        self._layt.setSpacing(defs.spacing)
-        self._layt.addWidget(self._tabbar)
-        self._layt.addWidget(self._stackedwidget)
-        self._layt.addLayout(self._button_layt)
-        self.setLayout(self._layt)
+        self.main_layout = qtutils.vbox(defs.margin, defs.spacing,
+                                        self.tab_bar, self.stack_widget,
+                                        self.button_layout)
+        self.setLayout(self.main_layout)
 
-        self.connect(self._tabbar, SIGNAL('currentChanged(int)'),
-                     self._stackedwidget.setCurrentIndex)
+        self.connect(self.tab_bar, SIGNAL('currentChanged(int)'),
+                     self.stack_widget.setCurrentIndex)
 
-        self.connect(self._stackedwidget, SIGNAL('currentChanged(int)'),
+        self.connect(self.stack_widget, SIGNAL('currentChanged(int)'),
                      self.update_widget)
 
         qtutils.connect_button(self.close_button, self.accept)
@@ -285,5 +293,5 @@ class PreferencesView(standard.Dialog):
         self.update_widget(0)
 
     def update_widget(self, idx):
-        widget = self._stackedwidget.widget(idx)
+        widget = self.stack_widget.widget(idx)
         widget.update_from_config()
